Attribute VB_Name = "mCore"
Option Explicit

'Types
    Public Type tPoint
        x As Long
        y As Long
    End Type

'Declares
    Public Declare Function GetTickCount Lib "kernel32" () As Long
    
    Public Declare Function GetPixel Lib "gdi32" (ByVal hdc As Long, ByVal x As Long, ByVal y As Long) As Long
    
    Public Declare Function MoveToEx Lib "gdi32" (ByVal iDC As Long, ByVal x As Long, ByVal y As Long, iPoint As tPoint) As Long
    Public Declare Function LineTo Lib "gdi32" (ByVal iDC As Long, ByVal x As Long, ByVal y As Long) As Long
    
    Public Declare Function DeleteObject Lib "gdi32" (ByVal iObject As Long) As Long
    Public Declare Function SelectObject Lib "gdi32" (ByVal iDC As Long, ByVal iObject As Long) As Long
    
    Public Declare Function CreateSolidBrush Lib "gdi32" (ByVal iColor As Long) As Long
    Public Declare Function CreatePen Lib "gdi32" (ByVal iStyle As Long, ByVal iWidth As Long, ByVal iColor As Long) As Long

'Data type definition
    Public Type tStar
        x As Single
        y As Single
        z As Single
    End Type

'The stars array
    Public StarCount As Long
    Public Star() As tStar

'Screen size
    Public ScreenW As Long
    Public ScreenH As Long
    
    Public OffsetX As Long
    Public OffsetY As Long

'Setup
    Public Speed As Single
    
    Public SpreadX As Single
    Public SpreadY As Single
    Public SpreadZ As Single

Public Function DrawPoint(DC As Long, x As Long, y As Long, iSize As Long, iColor As Long)
    Dim Pen As Long
    Dim Brush As Long
    
    Dim OldPen As Long
    Dim OldBrush As Long
    
    Dim TempPos As tPoint
    
    'Create pen
    Pen = CreatePen(0, iSize, iColor)
    OldPen = SelectObject(DC, Pen)
    
    'Create brush
    Brush = CreateSolidBrush(iColor)
    OldBrush = SelectObject(DC, Brush)
    
    'Draw point
    MoveToEx DC, x, y, TempPos
    LineTo DC, x, y
    
    'Restore brush
    SelectObject DC, OldBrush
    DeleteObject Brush
    
    'Restore pen
    SelectObject DC, OldPen
    DeleteObject Pen
End Function
Public Sub DrawStars(DC As Long)
    Dim A As Long
    
    Dim TempX As Long
    Dim TempY As Long
    Dim Trans As Single
    
    Dim Color As Long
    Dim tR As Long
    Dim tG As Long
    Dim tB As Long
    
    For A = 0 To StarCount
        With Star(A)
            'Get 2D (projected) position
            TempX = (.x / .z) + OffsetX
            TempY = (.y / .z) + OffsetY
            
            'Get transparency from distance (here ^ 0.3 smooths the transparent fading)
            Trans = (.z / SpreadZ) ^ 0.3
            
            'Get BG color
            Color = GetPixel(DC, TempX, TempY)
            
            'Split color into RGB values
            tR = (Color And &HFF)
            tG = (Color And &HFF00&) \ &H100&
            tB = (Color And &HFF0000) \ &H10000
            
            'Calculate the brightening factor
            Color = (1 - Trans) * 255
            
            'Calculate transparent color
            tR = (Trans * tR) + Color
            tG = (Trans * tG) + Color
            tB = (Trans * tB) + Color
            
            'Draw star (here ^ 5 smooths the size growing; 6 is the maximum size)
            DrawPoint DC, TempX, TempY, 1 + (1 - (.z / SpreadZ)) ^ 5 * 6, RGB(tR, tG, tB)
        End With
    Next
End Sub

Public Sub MoveStars()
    Dim A As Long
    
    For A = 0 To StarCount
        With Star(A)
            'Move towards screen
            .z = .z - Speed
            
            If .z <= 0 Then
                'Reset star when in front of screen
                .x = (Rnd - 0.5) * SpreadX
                .y = (Rnd - 0.5) * SpreadY
                .z = SpreadZ
            End If
        End With
    Next
End Sub


Public Sub SetupStars(iCount As Long)
    Dim A As Long
    
    'Create stars array
    StarCount = iCount
    ReDim Star(StarCount)
    
    'Set random positions
    For A = 0 To StarCount
        With Star(A)
            .x = (Rnd - 0.5) * SpreadX
            .y = (Rnd - 0.5) * SpreadY
            .z = Rnd * SpreadZ
        End With
    Next
End Sub


